/**
 * 
 */
package edu.jhu.cs.oose.elevel.AdvancedEditCard;

import com.trolltech.qt.QSignalEmitter;
import com.trolltech.qt.gui.QAbstractButton;
import com.trolltech.qt.gui.QButtonGroup;

/**
 * @author Jesse Yates
 *         ButtonGroup that can emit the DrawingType that was changed to
 */
public class DrawingButtonGroup extends QButtonGroup
{
	/**
	 * Signal that indicates which drawingType has been changed to
	 */
	public final QSignalEmitter.Signal1<DrawingType> typeChange = new QSignalEmitter.Signal1<DrawingType>();

	/**
	 * Default constructor
	 */
	public DrawingButtonGroup()
	{
		this.buttonClicked.connect(this, "changeType(QAbstractButton)");
	}

	/**
	 * Emits the type that has been changed to if the button is a DrawingButton
	 * 
	 * @param b
	 */
	public void changeType(QAbstractButton b)
	{
		if(b instanceof DrawingButton)
		{
			DrawingButton a = (DrawingButton) b;
			typeChange.emit(a.getType());
		}
	}

	/**
	 * Gets the type that has been currently set in the group
	 * 
	 * @return <code>DrawingType</code> that has been currently set
	 */
	public DrawingType getSetType()
	{
		for(QAbstractButton b : this.buttons())
		{
			if(b instanceof DrawingButton)
			{
				DrawingButton a = (DrawingButton) b;
				return a.getType();
			}
		}
		return null;
	}
}
