/**
 * DefaultCanvasScene.java
 * Group 7, Object Oriented Software Engineering
 * Nov 3, 2009
 */
package edu.jhu.cs.oose.elevel.AdvancedEditCard;

import com.trolltech.qt.QSignalEmitter;
import com.trolltech.qt.core.QRectF;
import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QColor;
import com.trolltech.qt.gui.QFocusEvent;
import com.trolltech.qt.gui.QFont;
import com.trolltech.qt.gui.QGraphicsItemInterface;
import com.trolltech.qt.gui.QGraphicsScene;
import com.trolltech.qt.gui.QGraphicsSceneMouseEvent;
import com.trolltech.qt.gui.QLineF;
import com.trolltech.qt.gui.QPen;
import com.trolltech.qt.gui.QTextCursor;
import com.trolltech.qt.gui.QUndoStack;

/**
 * @author Jesse Yates
 *         Underlying model for the the DrawingView. Deals with all the adding/removing of objects
 *         Currently VERY bad design - first concern is working implementation (conscious violation of OCP)
 */
public class DrawingGraphicsScene extends QGraphicsScene
{
	/** Whether or not it has been modified */
	private boolean isModified;

	/** Current width of the pen for shapes */
	private int myPenWidth;

	/** Current color of the pen for shapes */
	private QColor myPenColor;

	/** Whether or not text boxes added should be centered */
	private boolean centerText;
	// private DrawingType type;

	/** the last line drawn/being drawn */
	private DrawingLineItem drawingLine;

	/** The last circle drawn/being draw */

	private DrawingEllipseItem drawingCircle;

	/** the last square drawn/being drawn */
	private DrawingRectItem drawingSquare;

	/** Stack maintaining the undo/redo commands */
	private QUndoStack stack;

	/**
	 * Signals to indicate if there has been a mouse action
	 */
	private final QSignalEmitter.Signal1<QGraphicsSceneMouseEvent> drawMousePress = new QSignalEmitter.Signal1<QGraphicsSceneMouseEvent>();
	private final QSignalEmitter.Signal1<QGraphicsSceneMouseEvent> drawMouseHold = new QSignalEmitter.Signal1<QGraphicsSceneMouseEvent>();
	private final QSignalEmitter.Signal1<QGraphicsSceneMouseEvent> drawMouseRelease = new QSignalEmitter.Signal1<QGraphicsSceneMouseEvent>();

	/** Indicates if the element has gained focus */
	public final QSignalEmitter.Signal0 gainedFocus = new QSignalEmitter.Signal0();
	public final QSignalEmitter.Signal0 lostFocus = new QSignalEmitter.Signal0();

	/**
	 * Default Constructor
	 */
	public DrawingGraphicsScene()
	{
		this.stack = new QUndoStack();
		this.centerText = false;
		this.myPenColor = new QColor(QColor.black);
		this.myPenWidth = 5;
		this.isModified = false;
		this.setFont(new QFont("Times New Roman", 12, 50));
	}

	/**
	 * @return If the canvas has been modified
	 */
	public boolean isModified()
	{
		return isModified;
	}

	/**
	 * @param isModified
	 *        the isModified to set
	 */
	public void setModified(boolean isModified)
	{
		this.isModified = isModified;
	}

	/**
	 * Whether or not to center the text in the panel
	 * 
	 * @param b
	 *        <code>true</code> if it should be centered, <code>false</code> otherwise
	 */
	public void centerText(boolean b)
	{
		this.centerText = b;
	}

	/**
	 * Get the pen color
	 * 
	 * @return The <code>QColor</code> of the pen
	 */
	public QColor penColor()
	{
		return myPenColor;
	}

	/**
	 * Get the pen width
	 * 
	 * @return the pen width
	 */
	public int penWidth()
	{
		return myPenWidth;
	}

	/**
	 * Set the color of the pen
	 * 
	 * @param newColor
	 *        The <code>QColor</code> of the pen
	 */
	public void setPenColor(QColor newColor)
	{
		this.myPenColor = newColor;
	}

	/**
	 * Set the width of the pen
	 * 
	 * @param width
	 *        The new pen width
	 */
	public void setPenWidth(int width)
	{
		this.myPenWidth = width;

	}

	/**
	 * Handles mousePress events. Emits signal that is there is a left button click to any listening object/method
	 */
	protected void mousePressEvent(QGraphicsSceneMouseEvent event)
	{
		if(event.button() == Qt.MouseButton.LeftButton)
		{
			this.drawMousePress.emit(event);

		}
		super.mousePressEvent(event);
		update();
	}

	/**
	 * Handles mouseMove events. Emits signal that is there is a move to any listening object/method
	 */
	protected void mouseMoveEvent(QGraphicsSceneMouseEvent event)
	{
		this.drawMouseHold.emit(event);
		super.mouseMoveEvent(event);
		update();
	}

	/**
	 * Handles mouseRelease events. Emits signal that is there is a release to any listening object/method
	 */
	protected void mouseReleaseEvent(QGraphicsSceneMouseEvent event)
	{
		System.out.println("Mouse Release!");
		this.drawMouseRelease.emit(event);
		super.mousePressEvent(event);
		update();
		// TODO this is temporary, need to find where to implement
		setModified(true);
	}

	/**
	 * If there is already an object at the current location, does not do anything, passes on the mouse click to that
	 * element. Otherwise, creates a textbox at that location
	 * 
	 * @param event
	 *        the event that is being acted on
	 */
	private void textPressEvent(QGraphicsSceneMouseEvent event)
	{
		System.out.println("TextPressEvent");
		if(this.itemAt(event.scenePos()) == null)
		{
			System.out.println("Item is null!");
			DrawingTextItem text = new DrawingTextItem("");
			text.setFont(this.font());
			text.setTextInteractionFlags(Qt.TextInteractionFlag.TextEditorInteraction);
			text.setZValue(1000.0);
			text.lostFocus.connect(this, "itemLostFocus(DrawingTextItem)");

			if(!this.centerText)
				text.setPos(event.scenePos());
			else
				text.setPos(0, 0);

			System.out.println("Adding text at: " + text.pos());
			this.addEvent(new DrawItemCommand(text, this));
			// text.mousePressEvent(event);
			// text.mouseDoubleClickEvent(event);
		}
		else
		{
			System.out.println(this.itemAt(event.scenePos()));
			this.itemAt(event.scenePos()).mousePressEvent(event);
		}
		// super.mousePressEvent(event);
		event.accept();

	}

	/**
	 * If there is already an object at the current location, does not do anything, passes on the mouse click to that
	 * element. Otherwise, creates a line starting at that location
	 * 
	 * @param event
	 *        the event that is being acted on
	 */
	private void linePressEvent(QGraphicsSceneMouseEvent event)
	{
		System.out.println("line Press event");
		if(this.itemAt(event.scenePos()) != null)
		{
			System.out.println("Item is not null!");
			this.itemAt(event.scenePos()).mouseMoveEvent(event);// setFocus(Qt.FocusReason.MouseFocusReason);
		}
		else
		{
			System.out.println("Item is not null, adding new line!");
			this.drawingLine = new DrawingLineItem(new QLineF(event.scenePos(), event.scenePos()));
			this.drawingLine.setPen(new QPen(this.myPenColor, this.myPenWidth));
			this.addItem(drawingLine);
		}
		event.accept();
	}

	/**
	 * If there is already a line started, continues growing the line. Otherwise, just accepts the event
	 * 
	 * @param event
	 *        the event that is being acted on
	 */
	private void lineMoveEvent(QGraphicsSceneMouseEvent event)
	{
		// System.out.println("Line Mouse hold!");
		if(this.drawingLine != null)
		{
			// this.removeItem(this.drawingLine);
			QLineF newLine = new QLineF(this.drawingLine.line().p1(), event.scenePos());
			this.drawingLine.setLine(newLine);
		}
		event.accept();
	}

	/**
	 * Adds the line that is currently being drawn to the stack and draws the element
	 * 
	 * @param event
	 *        The event causing this action
	 */
	private void lineReleaseEvent(QGraphicsSceneMouseEvent event)
	{
		if(this.drawingLine != null)
		{
			this.removeItem(this.drawingLine);
			this.addEvent(new DrawItemCommand(this.drawingLine, this));
			this.drawingLine = null;
		}
		event.accept();
	}

	/**
	 * If there is already an object at the current location, does not do anything, passes on the mouse click to that
	 * element. Otherwise, creates a circle starting at that location
	 * 
	 * @param event
	 *        the event that is being acted on
	 */
	private void circlePressEvent(QGraphicsSceneMouseEvent event)
	{
		System.out.println("Circle Press event");
		if(this.itemAt(event.scenePos()) != null)
		{
			System.out.println("Item is not null!");
			this.itemAt(event.scenePos()).mouseMoveEvent(event);// setFocus(Qt.FocusReason.MouseFocusReason);
		}
		else
		{

			this.drawingCircle = new DrawingEllipseItem(event.scenePos().x(), event.scenePos().y(), 0, 0);
			this.drawingCircle.setPen(new QPen(this.myPenColor, this.myPenWidth));
			System.out.println("new circle:" + this.drawingCircle.rect());
			this.addItem(drawingCircle);
		}
		event.accept();
	}

	/**
	 * If there is already a circle started, continues growing the circle. Otherwise, just accepts the event
	 * 
	 * @param event
	 *        the event that is being acted on
	 */
	private void circleMoveEvent(QGraphicsSceneMouseEvent event)
	{
		// System.out.println("Circle Mouse hold!");
		if(this.drawingCircle != null)
		{
			double height = -(this.drawingCircle.rect().x() - event.scenePos().x());
			if(height == 0)
				height = 1;
			double width = -(this.drawingCircle.rect().y() - event.scenePos().y());
			if(width == 0)
				width = 1;
			QRectF newCircle = new QRectF(this.drawingCircle.rect().x(), this.drawingCircle.rect().y(), height, width);
			System.out.println("Circle parameters: " + newCircle);
			this.drawingCircle.setRect(newCircle);
		}
		event.accept();
	}

	/**
	 * Adds the circle that is currently being drawn to the stack and draws the element
	 * 
	 * @param event
	 *        The event causing this action
	 */
	private void circleReleaseEvent(QGraphicsSceneMouseEvent event)
	{
		if(this.drawingCircle != null)
		{
			this.removeItem(this.drawingCircle);
			this.addEvent(new DrawItemCommand(this.drawingCircle, this));
			this.drawingCircle = null;
		}
		event.accept();
	}

	/**
	 * If there is already an object at the current location, does not do anything, passes on the mouse click to that
	 * element. Otherwise, creates a square starting at that location
	 * 
	 * @param event
	 *        the event that is being acted on
	 */
	private void squarePressEvent(QGraphicsSceneMouseEvent event)
	{
		System.out.println("Circle Press event");
		if(this.itemAt(event.scenePos()) != null)
		{
			System.out.println("Item is not null!");
			this.itemAt(event.scenePos()).mouseMoveEvent(event);// setFocus(Qt.FocusReason.MouseFocusReason);
		}
		else
		{

			this.drawingSquare = new DrawingRectItem(event.scenePos().x(), event.scenePos().y(), 0, 0);
			this.drawingSquare.setPen(new QPen(this.myPenColor, this.myPenWidth));
			System.out.println("new line:" + this.drawingSquare.rect());
			this.addItem(drawingSquare);
		}
		event.accept();
	}

	/**
	 * If there is already a square started, continues growing the square. Otherwise, just accepts the event
	 * 
	 * @param event
	 *        the event that is being acted on
	 */
	private void squareMoveEvent(QGraphicsSceneMouseEvent event)
	{
		if(this.drawingSquare != null)
		{
			double height = -(this.drawingSquare.rect().x() - event.scenePos().x());
			if(height == 0)
				height = 1;
			double width = -(this.drawingSquare.rect().y() - event.scenePos().y());
			if(width == 0)
				width = 1;
			QRectF newRect = new QRectF(this.drawingSquare.rect().x(), this.drawingSquare.rect().y(), height, width);
			System.out.println("Rect parameters: " + newRect);
			this.drawingSquare.setRect(newRect);
		}
		event.accept();
	}

	/**
	 * Adds the square that is currently being drawn to the stack and draws the element
	 * 
	 * @param event
	 *        The event causing this action
	 */
	private void squareReleaseEvent(QGraphicsSceneMouseEvent event)
	{
		if(this.drawingSquare != null)
		{
			this.removeItem(this.drawingSquare);
			this.addEvent(new DrawItemCommand(this.drawingSquare, this));
			this.drawingSquare = null;
		}
		event.accept();
	}

	/**
	 * Slot for the lostFocus slot that <code>CardTextItem</code> has. Useful for removing a if there is no text in the
	 * box
	 * 
	 * @param item
	 */
	public void itemLostFocus(DrawingTextItem item)
	{
		QTextCursor cursor = item.textCursor();
		cursor.clearSelection();
		item.setTextCursor(cursor);

		if(item.toPlainText().length() == 0)
		{
			this.removeItem(item);
			System.out.println("Removing item!");
		}
	}

	/**
	 * Set the current type based on the button that was pressed. Connects the proper slots to those objects
	 * 
	 * @param buttonPressed
	 *        the button pressed - needs to be a DrawingRadioButton
	 */
	public void setType(DrawingType type)
	{
		System.out.println("Setting type to: " + type);
		this.drawMousePress.disconnect(this);
		if(type.equals(DrawingType.TEXT))
		{
			this.drawMousePress.disconnect(this);
			this.drawMousePress.connect(this, "textPressEvent(QGraphicsSceneMouseEvent)");
		}
		else if(type.equals(DrawingType.LINE))
		{
			this.drawMousePress.disconnect(this);
			this.drawMousePress.connect(this, "linePressEvent(QGraphicsSceneMouseEvent)");
			this.drawMouseHold.connect(this, "lineMoveEvent(QGraphicsSceneMouseEvent)");
			this.drawMouseRelease.connect(this, "lineReleaseEvent(QGraphicsSceneMouseEvent)");

		}
		else if(type.equals(DrawingType.CIRCLE))
		{
			this.drawMousePress.disconnect(this);
			this.drawMousePress.connect(this, "circlePressEvent(QGraphicsSceneMouseEvent)");
			this.drawMouseHold.connect(this, "circleMoveEvent(QGraphicsSceneMouseEvent)");
			this.drawMouseRelease.connect(this, "circleReleaseEvent(QGraphicsSceneMouseEvent)");

		}
		else if(type.equals(DrawingType.SQUARE))
		{
			this.drawMousePress.disconnect(this);
			this.drawMousePress.connect(this, "squarePressEvent(QGraphicsSceneMouseEvent)");
			this.drawMouseHold.connect(this, "squareMoveEvent(QGraphicsSceneMouseEvent)");
			this.drawMouseRelease.connect(this, "squareReleaseEvent(QGraphicsSceneMouseEvent)");

		}
		else
		{
			// this.drawMousePress.disconnect();

		}
		// this.type = type;
	}

	/**
	 * Undoes the last element on the stack
	 */
	public void undo()
	{
		if(this.hasFocus())
		{
			if(this.stack.canUndo())
				this.stack.undo();
			if(!this.stack.canUndo())
				this.setModified(false);
		}
	}

	/**
	 * Redoes the last element on the stack
	 */
	public void redo()
	{
		if(this.hasFocus())
			if(this.stack.canRedo())
				this.stack.redo();
	}

	/**
	 * /**
	 * Add an event to the stack
	 * 
	 * @param command
	 *        To be added. only accepted if it is a valid command
	 * @return <code>true</code> if the item is pushed on the stack, false otherwise
	 */
	public boolean addEvent(DrawCommand command)
	{
		if(command.isValid())
		{
			this.stack.push(command);
			return true;
		}
		return false;
	}

	public String writeout()
	{
		StringBuffer output = new StringBuffer();
		for(QGraphicsItemInterface i : this.items())
		{
			System.out.println("Item i:" + i);
			/*
			if(i instanceof CardTextItem)
				output.append(i.getClass().getName() + ",rectangle:" + i.boundingRect() + ", con");
			else
			{
				output.append("Card" + ", " + i.boundingRect() + ", contents:");
			}*/
		}

		return output.toString();

	}

	protected void focusInEvent(QFocusEvent event)
	{
		super.focusInEvent(event);
		this.gainedFocus.emit();
	}

	protected void focusOutEvent(QFocusEvent event)
	{
		super.focusOutEvent(event);
		this.lostFocus.emit();
	}

}
