/**
 * CardAreaGUI.java
 * Group 7, Object Oriented Software Engineering
 * October 20, 2009
 */
package edu.jhu.cs.oose.elevel;

import com.trolltech.qt.core.QSize;
import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QGridLayout;
import com.trolltech.qt.gui.QMainWindow;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QTextEdit;
import com.trolltech.qt.gui.QWidget;
import edu.jhu.cs.oose.elevel.dataModel.Card;
import edu.jhu.cs.oose.elevel.utilities.ViewState;

/**
 * Uses the basic interface class QWidget to design the <code>Card</code> GUI
 * 
 * @author Pablo Lee
 */
public class CardAreaGUI extends QWidget
{
	private ViewState state;

	/** The QLayout of the card area */
	QGridLayout gridLayout;

	/** The minimum size of the card area */
	final static int minWidth = 350;
	final static int minHeight = 200;

	/**
	 * Initializes the basic layout of a <code>Card</code>, including previous and next card buttons
	 * 
	 * @param window
	 *        the <code>MainWindow</code> of E-Level
	 */
	public CardAreaGUI(QMainWindow window)
	{
		state = ViewState.INSTANCE;
		state.cardSelectedInTree.connect(this, "displayCard(Card)");

		this.gridLayout = new QGridLayout(this);
		setLayout(this.gridLayout);

		if(state.getNumDecks() == 0)
		{
			displayEmptyCard();
		}
		else
		{
			displayCard(state.getCurrentCard());
		}

		QPushButton previousButton = new QPushButton(tr("Previous"), this);
		previousButton.clicked.connect(state, "previousCard()");
		this.gridLayout.addWidget(previousButton, 1, 0, Qt.AlignmentFlag.AlignLeft);

		QPushButton nextButton = new QPushButton(tr("Next"), this);
		nextButton.clicked.connect(state, "nextCard()");
		this.gridLayout.addWidget(nextButton, 1, 2, Qt.AlignmentFlag.AlignRight);
	}

	/**
	 * Displays the current card
	 * 
	 * @param currentCard
	 *        the <code>Card</code> to be displayed in the card area
	 */
	private void displayCard(Card currentCard)
	{
		if(currentCard != null)
		{
			QTextEdit question = new QTextEdit();
			question.setText(currentCard.getQuestion());
			question.setReadOnly(true);
			question.setMinimumSize(new QSize(minWidth, minHeight));
			this.gridLayout.addWidget(question, 0, 0);

			QTextEdit answer = new QTextEdit();
			answer.setText(currentCard.getAnswer());
			answer.setReadOnly(true);
			answer.setMinimumSize(new QSize(minWidth, minHeight));
			this.gridLayout.addWidget(answer, 2, 0);

			QTextEdit additionalInfo = new QTextEdit();
			additionalInfo.setText(currentCard.getAdditionalInfo());
			additionalInfo.setReadOnly(true);
			additionalInfo.setMinimumSize(new QSize(minWidth, minHeight));
			this.gridLayout.addWidget(additionalInfo, 0, 2);

			QTextEdit hint = new QTextEdit();
			hint.setText(currentCard.getHint());
			hint.setReadOnly(true);
			hint.setMinimumSize(new QSize(minWidth, minHeight));
			this.gridLayout.addWidget(hint, 2, 2);
		}
		else
		{
			displayEmptyCard();
		}
	}

	/**
	 * Displays an empty <code>Card</code> in the card area
	 */
	public void displayEmptyCard()
	{
		displayCard(new Card());
	}
}
