/**
 * EditCardWindow.java
 * Group 7, Object Oriented Software Engineering
 * October 10, 2009
 */
package edu.jhu.cs.oose.elevel.editing;

import java.util.Map;
import edu.jhu.cs.oose.elevel.dataModel.Card;
import edu.jhu.cs.oose.elevel.utilities.ViewState;

/**
 * The underlying model class that handles editing any features of a <code>Card</code>
 * 
 * @author Jesse Yates, H. Parker Shelton
 */
public class EditCardWindow
{
	/** The <code>Card</code> currently being edited */
	private Card card;

	/** The <code>ToolBar</code> currently being used */
	private DefaultToolBar editToolbar;

	/**
	 * Constructor for the editCard model, creates the default {@link ToolBar} to be used
	 * 
	 * @param c
	 *        the {@link Card} currently selected
	 */
	public EditCardWindow(Card c)
	{
		setCard(c);
		this.editToolbar = new DefaultToolBar();
	}

	/**
	 * Constructor for the editCard model
	 * 
	 * @param c
	 *        the <code>Card</code> to be edited
	 * @param currentToolbar
	 *        the <code>ToolBar</code> to edit with
	 */
	public EditCardWindow(Card c, ToolBar currentToolbar)
	{
		this(c);
		// TODO change this line to a different toolbar
	}

	/**
	 * Returns the <code>Card</code> currently being edited
	 * 
	 * @return a {@link Card} representing the card currently being edited
	 */
	public Card getCard()
	{
		return card;
	}

	/**
	 * Set the <code>Card</code> being currently edited to the specified {@link Card}
	 * 
	 * @param c
	 *        a <code>Card</code> representing the card currently being edited
	 */
	public void setCard(Card c)
	{
		if(c == null)
			this.card = new Card();
		else
			this.card = c;
	}

	/**
	 * Return the {@link ToolBar} currently associated with this <code>EditCardWindow</code>
	 * 
	 * @return the {@link ToolBar} currently associated with this <code>EditCardWindow</code>
	 */
	public DefaultToolBar getToolbar()
	{
		return editToolbar;
	}

	/**
	 * Set the {@link ToolBar} currently being used to the specified {@link ToolBar}
	 * 
	 * @param e
	 *        the {@link ToolBar} to associate with this <code>EditCardWindow</code>
	 */
	public void setToolbar(DefaultToolBar e)
	{
		this.editToolbar = e;
	}

	/**
	 * Saves a new card to the deck with the properties passed in
	 * 
	 * @param question
	 *        the question on the flash card
	 * @param answer
	 *        the answer to the question on the flash card
	 * @param hint
	 *        the user specified hint to the answer
	 * @param addinfo
	 *        any additional information given
	 * @param pref
	 *        preferences
	 */
	public void save(String question, String answer, String hint, String addinfo, Map<String, Boolean> pref)
	{
		this.card.setValues(question, answer, hint, addinfo, pref);
		ViewState.INSTANCE.getCurrentDeck().setHasChanged(true);
	}
}
