/**
 * SerializationTest.java
 * Group 7, Object Oriented Software Engineering
 * November 3, 2009
 */
package edu.jhu.cs.oose.elevel.junit;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import junit.framework.TestCase;
import edu.jhu.cs.oose.elevel.dataModel.Card;
import edu.jhu.cs.oose.elevel.dataModel.Deck;
import edu.jhu.cs.oose.elevel.dataModel.TestStat;

/**
 * Class for testing the serialization of objects to be stored on file.
 * 
 * @author H. Parker Shelton
 */
public class SerializationTest extends TestCase
{
	private String exportLocation;
	private List<Deck> openDecks;
	private List<String> serializations;

	/**
	 * Initializes the test.
	 * 
	 * @throws IOException
	 */
	protected void setUp() throws IOException
	{
		exportLocation = "/Users/Parker/Documents/E-Level";
		openDecks = new ArrayList<Deck>();
		serializations = new ArrayList<String>();

		Deck deck = new Deck("");
		deck.setDiskLocation(exportLocation + "/" + deck.getName() + ".elvl");
		openDecks.add(deck);
		deck = new Deck("Empty");
		deck.setDiskLocation(exportLocation + "/" + deck.getName() + ".elvl");
		openDecks.add(deck);
		deck = new Deck("NotEmpty");
		deck.setDiskLocation(exportLocation + "/" + deck.getName() + ".elvl");
		deck.addCard(new Card("Not", "Empty", "", ""));
		deck.addCard(new Card("Also", "Not", "Empty", "Card"));
		deck.addTestStat(new TestStat(10, 20, new Date(100000), 500));
		deck.addTestStat(new TestStat(10, 30, new Date(50000000), 500));
		deck.addTestStat(new TestStat(1, 10000, new Date(999999999), 0));
		openDecks.add(deck);

		for(Deck d : openDecks)
		{
			d.setHasChanged(false);
			serializations.add(serialize(d));
		}
	}

	/**
	 * Tests writing of <code>Deck</code>s (and <code>Card</code>s) to file
	 */
	public void testWrite()
	{
		for(Deck d : openDecks)
			d.writeToDisk(null, exportLocation + "/" + d.getName() + ".elvl");

		assertTrue((new File(exportLocation + "/Default Deck.elvl")).exists());
		assertTrue((new File(exportLocation + "/Empty.elvl")).exists());
		assertTrue((new File(exportLocation + "/NotEmpty.elvl")).exists());
	}

	/**
	 * Tests reading of <code>Deck</code>s (and <code>Card</code>s) to file
	 * 
	 * @throws IOException
	 */
	public void testRead() throws IOException
	{
		Deck d = Deck.readFromDisk(null, exportLocation + "/Default Deck.elvl");
		assertEquals(serialize(d), serializations.get(0));
		assertTrue(d.equals(openDecks.get(0)));

		d = Deck.readFromDisk(null, exportLocation + "/Empty.elvl");
		assertEquals(serialize(d), serializations.get(1));
		assertTrue(d.equals(openDecks.get(1)));

		d = Deck.readFromDisk(null, exportLocation + "/NotEmpty.elvl");
		assertEquals(serialize(d), serializations.get(2));
		assertTrue(d.equals(openDecks.get(2)));
	}

	/**
	 * Auxiliary function used during tests to serialize <code>Deck</code>s to <code>String</code>s
	 * 
	 * @param d
	 *        a <code>Deck</code> to be serialized
	 * @return a <code>String</code> representing the byte array of an output stream.
	 * @throws IOException
	 */
	private String serialize(Deck d) throws IOException
	{
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		ObjectOutputStream oos = new ObjectOutputStream(baos);
		oos.writeObject(d);
		oos.close();
		return new String(baos.toByteArray());
	}
}
