/**
 * ELevelChallengeWindow.java
 * Group 7, Object Oriented Software Engineering
 * November 6, 2009
 */
package edu.jhu.cs.oose.elevel.testTaking;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Random;
import edu.jhu.cs.oose.elevel.dataModel.Card;
import edu.jhu.cs.oose.elevel.dataModel.Deck;

/**
 * The class representing the E-Level Challenge Quiz functionality and display. Extends TestWindow and overrides
 * nextCard().
 * 
 * @author Greg Anderson
 */
public class ELevelChallengeWindow extends TestWindow
{
	/**
	 * A map between Cards and a probability for selecting a particular card during the quiz.
	 */
	private Map<Card, Float> eLvlChallengeDeck;

	/**
	 * Auxilliary storage for the card probabilities.
	 */
	private List<Float> probability = new ArrayList<Float>();
	/**
	 * Auxilliary storage for the cards.
	 */
	private List<Card> cardAssociation = new ArrayList<Card>();

	/**
	 * Instantiates card/probability associations and sets up the quiz.
	 * 
	 * @param currentDeck
	 *        A reference to the deck that current quiz is being run on.
	 */
	public ELevelChallengeWindow(Deck currentDeck)
	{
		super(currentDeck);

		eLvlChallengeDeck = currentDeck.getELevelChallengeDeck();

		// Transfer cards and probabilities to the auxilliary structures.
		float cummulativeSum = 0;
		for(Card card : eLvlChallengeDeck.keySet())
		{
			cummulativeSum += eLvlChallengeDeck.get(card);
			probability.add(cummulativeSum);
			cardAssociation.add(card);
		}
	}

	/**
	 * Probabilistically selects the next <code>Card</code>.
	 * 
	 * @return <code>true</code> if there is a next <code>Card</code>, <code>false</code> if the test is over
	 */
	public boolean nextCard()
	{
		Random rand = new Random();

		double selection = rand.nextDouble();

		for(int i = 0; i < probability.size(); i++)
		{
			if(selection < probability.get(i))
			{
				this.deck.setCurrentCard(cardAssociation.get(i));
				break;
			}
		}

		return true;
	}
}
